import type { RouteLocationNormalized, RouteRecordNormalized } from 'vue-router';
import type { App, Component } from 'vue';

import { intersectionWith, isEqual, mergeWith, unionWith } from 'lodash-es';
import { unref } from 'vue';
import { isArray, isObject } from '/@/utils/is';

export const noop = () => {};

/**
 * @description:  Set ui mount node
 */
export function getPopupContainer(node?: HTMLElement): HTMLElement {
  return (node?.parentNode as HTMLElement) ?? document.body;
}

/**
 * Add the object as a parameter to the URL
 * @param baseUrl url
 * @param obj
 * @returns {string}
 * eg:
 *  let obj = {a: '3', b: '4'}
 *  setObjToUrlParams('www.baidu.com', obj)
 *  ==>www.baidu.com?a=3&b=4
 */
export function setObjToUrlParams(baseUrl: string, obj: any): string {
  let parameters = '';
  for (const key in obj) {
    parameters += key + '=' + encodeURIComponent(obj[key]) + '&';
  }
  parameters = parameters.replace(/&$/, '');
  return /\?$/.test(baseUrl) ? baseUrl + parameters : baseUrl.replace(/\/?$/, '?') + parameters;
}

/**
 * Recursively merge two objects.
 * 递归合并两个对象。
 *
 * @param source The source object to merge from. 要合并的源对象。
 * @param target The target object to merge into. 目标对象，合并后结果存放于此。
 * @param mergeArrays How to merge arrays. Default is "replace".
 *        如何合并数组。默认为replace。
 *        - "union": Union the arrays. 对数组执行并集操作。
 *        - "intersection": Intersect the arrays. 对数组执行交集操作。
 *        - "concat": Concatenate the arrays. 连接数组。
 *        - "replace": Replace the source array with the target array. 用目标数组替换源数组。
 * @returns The merged object. 合并后的对象。
 */
export function deepMerge<T extends object | null | undefined, U extends object | null | undefined>(
  source: T,
  target: U,
  mergeArrays: 'union' | 'intersection' | 'concat' | 'replace' = 'replace',
): T & U {
  if (!target) {
    return source as T & U;
  }
  if (!source) {
    return target as T & U;
  }
  return mergeWith({}, source, target, (sourceValue, targetValue) => {
    if (isArray(targetValue) && isArray(sourceValue)) {
      switch (mergeArrays) {
        case 'union':
          return unionWith(sourceValue, targetValue, isEqual);
        case 'intersection':
          return intersectionWith(sourceValue, targetValue, isEqual);
        case 'concat':
          return sourceValue.concat(targetValue);
        case 'replace':
          return targetValue;
        default:
          throw new Error(`Unknown merge array strategy: ${mergeArrays as string}`);
      }
    }
    if (isObject(targetValue) && isObject(sourceValue)) {
      return deepMerge(sourceValue, targetValue, mergeArrays);
    }
    return undefined;
  });
}

export function openWindow(
  url: string,
  opt?: { target?: TargetContext | string; noopener?: boolean; noreferrer?: boolean },
) {
  const { target = '__blank', noopener = true, noreferrer = true } = opt || {};
  const feature: string[] = [];

  noopener && feature.push('noopener=yes');
  noreferrer && feature.push('noreferrer=yes');

  window.open(url, target, feature.join(','));
}

// dynamic use hook props
export function getDynamicProps<T extends Record<string, unknown>, U>(props: T): Partial<U> {
  const ret: Recordable = {};

  Object.keys(props).map((key) => {
    ret[key] = unref((props as Recordable)[key]);
  });

  return ret as Partial<U>;
}

export function getRawRoute(route: RouteLocationNormalized): RouteLocationNormalized {
  if (!route) return route;
  const { matched, ...opt } = route;
  return {
    ...opt,
    matched: (matched
      ? matched.map((item) => ({
          meta: item.meta,
          name: item.name,
          path: item.path,
        }))
      : undefined) as RouteRecordNormalized[],
  };
}

// https://github.com/vant-ui/vant/issues/8302
type EventShim = {
  new (...args: any[]): {
    $props: {
      onClick?: (...args: any[]) => void;
    };
  };
};

export type WithInstall<T> = T & {
  install(app: App): void;
} & EventShim;

export type CustomComponent = Component & { displayName?: string };

export const withInstall = <T extends CustomComponent>(component: T, alias?: string) => {
  (component as Record<string, unknown>).install = (app: App) => {
    const compName = component.name || component.displayName;
    if (!compName) return;
    app.component(compName, component);
    if (alias) {
      app.config.globalProperties[alias] = component;
    }
  };
  return component as WithInstall<T>;
};


const filterChildHierarchy = (hierarchies, parentHierarchy, key) => {
  const children = hierarchies.filter(hierarchy => hierarchy[key] === parentHierarchy.id);
  if (children.length > 0) {
    children.forEach(child => {
      child.children = filterChildHierarchy(hierarchies, child, key);
    })
  }
  children.sort((a, b) => a.sortOrder - b.sortOrder);
  return children;
}
export const allHierarchy2tree = (hierarchies, key) => {
  hierarchies.forEach(hierarchy => {
    hierarchy.key = hierarchy.id;
  })
  const root = hierarchies.filter(hierarchy => hierarchy[key] === 0 || typeof hierarchy[key] === 'undefined');
  root.forEach(hierarchy => {
    hierarchy.key = hierarchy.id;
    hierarchy.children = filterChildHierarchy(hierarchies, hierarchy, key);
  })
  root.sort((a, b) => a.sortOrder - b.sortOrder);
  return root;
}

export const handleFetch = (results) => {
  results.forEach(result => {
    if (result.children && result.children.length) {
      result.children = handleFetch(result.children);
    } else {
      delete result.children;
    }
  })
  return results;
}


export const camelToUnderScore = (str) => {
  let result = '';
  let isLastUpperCase = false;

  for (let i = 0; i < str.length; i++) {
    let char = str[i];

    if (char === char.toUpperCase() && i !== 0) {
      isLastUpperCase = true;
    } else {
      isLastUpperCase = false;
    }

    if (isLastUpperCase) {
      result += '_' + char.toLowerCase();
    } else {
      result += char;
    }
  }

  return result;
}

/// 生成规格组合
export function generateCombinations(specs: {[key: string]: string[] }) {
  // 获取所有的规格名称（键）
  const keys = Object.keys(specs);
  // 初始化结果数组
  let result = [{}];

  // 对于每一个规格
  for (let i = 0; i < keys.length; i++) {
    const key = keys[i];
    const specValues = specs[key];
    const temp: any[] = [];

    // 对于结果中的每一个组合
    for (let j = 0; j < result.length; j++) {
      const res = result[j];

      // 对于该规格的每一个值
      for (let k = 0; k < specValues.length; k++) {
        const specValue = specValues[k];
        // 复制当前的组合，并添加新的规格值
        const newRes = Object.assign({}, res);
        newRes[key] = specValue;
        // 将新的组合添加到临时数组中
        temp.push(newRes);
      }
    }
    // 更新结果数组
    result = temp;
  }

  // 返回所有的组合
  return result;
}
